@@ -39,13 +39,14 @@ VParseGrammar*	VParseGrammar::s_grammarp = NULL;
 //*************************************************************************
 
 VParse::VParse(VFileLine* filelinep, av* symsp,
-	       bool sigParser, bool useUnreadbackFlag, bool useProtected)
+	       bool sigParser, bool useUnreadbackFlag, bool useProtected, bool usePinselects)
     : m_syms(filelinep, symsp)
 {
     m_inFilelinep = filelinep;
     m_sigParser = sigParser;
     m_useUnreadback = useUnreadbackFlag;
     m_useProtected = useProtected;
+    m_usePinselects = usePinselects;
     m_debug = 0;
     m_lexp = new VParseLex(this);
     m_grammarp = new VParseGrammar(this);
